import { MutableRefObject, ReactElement } from "react";
type AnyRecord = Record<string, unknown>;
export type ValidatorResult<ErrorMessage = string> = ErrorMessage | void | Promise<ErrorMessage | void>;
export type Validator<Value, ErrorMessage = string> = (value: Value) => ValidatorResult<ErrorMessage>;
export type FormStatus = "untouched" | "editing" | "submitting" | "submitted";
export type Strategy = "onChange" | "onSuccess" | "onBlur" | "onSuccessOrBlur" | "onSubmit";
export type FieldState<Value, ErrorMessage = string> = {
    value: Value;
    validating: boolean;
    valid: boolean;
    error: ErrorMessage | undefined;
};
export type FormConfig<Values extends AnyRecord, ErrorMessage = string> = {
    [N in keyof Values]: {
        initialValue: Values[N] | (() => Values[N]);
        strategy?: Strategy;
        debounceInterval?: number;
        equalityFn?: (valueBeforeValidate: Values[N], valueAfterValidate: Values[N]) => boolean;
        sanitize?: (value: Values[N]) => Values[N];
        validate?: (value: Values[N], helpers: {
            focusField: (name: keyof Values) => void;
            getFieldState: <N extends keyof Values>(name: N, options?: {
                sanitize?: boolean;
            }) => FieldState<Values[N], ErrorMessage>;
        }) => ValidatorResult<ErrorMessage>;
    };
};
export type Form<Values extends AnyRecord, ErrorMessage = string> = {
    formStatus: FormStatus;
    Field: (<N extends keyof Values>(props: {
        name: N;
        children: (props: FieldState<Values[N], ErrorMessage> & {
            ref: MutableRefObject<any>;
            onChange: (value: Values[N]) => void;
            onBlur: () => void;
            focusNextField: () => void;
        }) => ReactElement | null;
    }) => ReactElement | null) & {
        displayName?: string;
    };
    FieldsListener: (<N extends keyof Values>(props: {
        names: N[];
        children: (states: {
            [N1 in N]: FieldState<Values[N1], ErrorMessage>;
        }) => ReactElement | null;
    }) => ReactElement | null) & {
        displayName?: string;
    };
    getFieldState: <N extends keyof Values>(name: N, options?: {
        sanitize?: boolean;
    }) => FieldState<Values[N], ErrorMessage>;
    setFieldValue: <N extends keyof Values>(name: N, value: Values[N], options?: {
        validate?: boolean;
    }) => void;
    setFieldError: (name: keyof Values, error?: ErrorMessage) => void;
    focusField: (name: keyof Values) => void;
    resetField: (name: keyof Values, options?: {
        feedbackOnly?: boolean;
    }) => void;
    sanitizeField: (name: keyof Values) => void;
    validateField: (name: keyof Values) => Promise<ErrorMessage | void>;
    listenFields: <N extends keyof Values>(names: N[], listener: (states: {
        [N1 in N]: FieldState<Values[N1], ErrorMessage>;
    }) => void) => () => void;
    resetForm: (options?: {
        feedbackOnly?: boolean;
    }) => void;
    submitForm: (onSuccess: (values: Partial<Values>) => Promise<unknown> | void, onFailure?: (errors: Partial<Record<keyof Values, ErrorMessage>>) => Promise<unknown> | void, options?: {
        avoidFocusOnError?: boolean;
    }) => void;
};
export declare const combineValidators: <Value, ErrorMessage = string>(...validators: (false | Validator<Value, ErrorMessage>)[]) => Validator<Value, ErrorMessage>;
export declare const hasDefinedKeys: <T extends AnyRecord, K extends keyof T = keyof T>(object: T, keys: K[]) => object is T & { [K1 in K]-?: Exclude<T[K1], undefined>; };
export declare const useForm: <Values extends AnyRecord, ErrorMessage = string>(fields: FormConfig<Values, ErrorMessage>) => Form<Values, ErrorMessage>;
export {};
